default_manage_dict = {
    'project_name': 'Project',
    'help_text': "This is the {project_name} interactive shell",
    'shell': {  # Preferences for 'manage shell'
        'ipython_auto_reload': True,
        'banner': {  # Banner is the message printed on top of console
            'enabled': True,  # It can be disabled
            # Here it goes the message
            'message': "Manage Interactive Shell! edit your `manage.yml`",
        },
        'auto_import': {  # Objects to be auto imported to shell context
            'display': True,  # Weather to print all a list of all objects
            'objects': {},
        },
        'readline_enabled': False,  # MacOS has no readline support
        'init_script': 'print("Starting interactive shell!")',
    },
    'function_commands': [],
    'click_commands': [],
    'inline_commands': [],
}



import builtins
import pkgutil
from importlib import import_module
from inspect import getmembers

import click
from click.core import BaseCommand


def add_click_commands(module, cli, command_dict, namespaced):
    """Loads all click commands"""
    module_commands = [
        item for item in getmembers(module) if isinstance(item[1], BaseCommand)
    ]
    options = command_dict.get("config", {})
    namespace = command_dict.get("namespace")
    for name, function in module_commands:
        f_options = options.get(name, {})
        command_name = f_options.get("name", getattr(function, "name", name))
        if namespace:
            command_name = "{}_{}".format(namespace, command_name)
        elif namespaced:
            module_namespace = module.__name__.split(".")[-1]
            command_name = "{}_{}".format(module_namespace, command_name)
        function.short_help = f_options.get("help_text", function.short_help)
        cli.add_command(function, name=command_name)


def handle_option_and_arg_data(data):
    data = data or {}
    if "type" in data:
        data["type"] = getattr(builtins, data["type"])
    if "help_text" in data:
        data["help"] = data.pop("help_text")
    return data


def handle_options_and_args(_command, arguments, options):

    for argument in arguments:
        if isinstance(argument, dict):
            _command = click.argument(
                list(argument.keys())[0],
                **handle_option_and_arg_data(list(argument.values())[0])
            )(_command)
        else:
            _command = click.argument(argument)(_command)

    if isinstance(options, dict):
        for name, data in options.items():
            data = handle_option_and_arg_data(data)
            _command = click.option(name, **data)(_command)
    else:
        for name in options:
            _command = click.option(name)(_command)
    return _command


def make_command_from_function(function, options, help_text=None, arguments=None):  # noqa: E501
    if help_text:
        function.__doc__ = help_text

    function = click.command()(function)
    function = handle_options_and_args(function, arguments, options)
    return function


def make_command_from_string(code, cmd_context, options, help_text=None, arguments=None):  # noqa: E501
    def _command(*args, **kwargs):
        exec(code, cmd_context, kwargs)

    return make_command_from_function(_command, options, help_text, arguments)


def get_context(context):
    return {item: import_module(item) for item in context}


def load_commands(cli, manage_dict):
    """Loads the commands defined in manage file"""
    namespaced = manage_dict.get("namespaced")

    # get click commands
    commands = manage_dict.get("click_commands", [])
    for command_dict in commands:
        root_module = import_module(command_dict["module"])
        group = cli.manage_groups.get(command_dict.get("group"), cli)
        if getattr(root_module, "__path__", None):
            # This is a package
            iter_modules = pkgutil.iter_modules(
                root_module.__path__, prefix=root_module.__name__ + "."
            )
            submodules_names = [item[1] for item in iter_modules]
            submodules = [import_module(name) for name in submodules_names]
            for module in submodules:
                add_click_commands(module, group, command_dict, namespaced)
        else:
            # a single file module
            add_click_commands(root_module, group, command_dict, namespaced)

    # get inline commands
    commands = manage_dict.get("inline_commands", [])
    for command_dict in commands:
        name = command_dict["name"]
        help_text = command_dict.get("help_text")
        options = command_dict.get("options", {})
        arguments = command_dict.get("arguments", {})
        context = command_dict.get("context", [])
        code = command_dict["code"]
        group = cli.manage_groups.get(command_dict.get("group"), cli)
        group.add_command(
            make_command_from_string(
                code=code,
                cmd_context=get_context(context),
                options=options,
                arguments=arguments,
                help_text=help_text,
            ),
            name=name,
        )

    # get function commands
    commands = manage_dict.get("function_commands", [])
    for command_dict in commands:
        name = command_dict["name"]
        help_text = command_dict.get("help_text")
        options = command_dict.get("options", {})
        arguments = command_dict.get("arguments", {})
        module_path, function_name = parse_function_path(
            command_dict["function"]
        )
        function = getattr(import_module(module_path), function_name)
        group = cli.manage_groups.get(command_dict.get("group"), cli)
        group.add_command(
            make_command_from_function(
                function=function,
                options=options,
                arguments=arguments,
                help_text=help_text,
            ),
            name=name,
        )


def parse_function_path(function_path):
    """Util to parse path to functions."""
    path_components = function_path.split(".")
    if len(path_components) > 2:
        function_name = path_components.pop()
        module_path = ".".join(path_components)
        return module_path, function_name
    return path_components[0], path_components[1]


def load_command_sources(manager, manage_dict):
    manage_sources = manage_dict.get("command_sources", [])
    for source_data in manage_sources:
        if isinstance(source_data, dict):
            source = import_module(source_data["name"])
            if callable(source):
                args = source_data.get("args", [])
                kwargs = source_data.get("kwargs", {})
                source = source(*args, **kwargs)
            manager.add_source(source)
        else:
            source = import_module(source_data)
            manager.add_source(source() if callable(source) else source)


def load_groups(cli, manage_dict):
    cli.manage_groups = {}
    groups = manage_dict.get("groups")
    if not groups:
        return
    is_dict = isinstance(groups[0], dict)
    for group in groups:
        if is_dict:
            for group_name, data in group.items():
                data = data or {}
                if "help_text" in data:
                    data["help"] = data.pop("help_text")
                cli.manage_groups[group_name] = cli.group(
                    name=group_name, **data
                )(lambda: None)
        else:
            cli.manage_groups[group] = cli.group(name=group)(lambda: None)



# coding: utf-8
import pkgutil
import sys
from importlib import import_module


def get_name(obj, default):
    default = default.split(".")[0]
    return getattr(obj, "__name__", default)


def import_submodules(name, submodules=None):
    """Import all submodules for a package/module name"""
    sys.path.insert(0, name)
    if submodules:
        for submodule in submodules:
            import_module("{0}.{1}".format(name, submodule))
    else:
        for item in pkgutil.walk_packages([name]):
            import_module("{0}.{1}".format(name, item[1]))


def import_objects(manage_dict):
    auto_import = {}
    auto_scripts = []
    import_dict = manage_dict.get("shell", {}).get("auto_import", {})
    object_list = import_dict.get("objects", [])
    if isinstance(object_list, dict):
        for name, spec in object_list.items():
            _obj = import_module(name)
            if spec:
                if "init" in spec:
                    init = spec["init"]
                    if isinstance(init, dict):
                        method_name = init.keys()[0]
                        args = (init[method_name] or {}).get("args", [])
                        kwargs = (init[method_name] or {}).get("kwargs", {})
                    else:
                        method_name = init
                        args = []
                        kwargs = {}
                    getattr(_obj, method_name)(*args, **kwargs)
                spec_as = spec.get("as", get_name(_obj, name))
                if not isinstance(spec_as, list):
                    spec_as = [spec_as]
                for as_name in spec_as:
                    auto_import[as_name] = _obj

                if "init_script" in spec:
                    auto_scripts.append(spec["init_script"])
                if "submodules" in spec:
                    submodules = spec["submodules"]
                    if isinstance(submodules, list):
                        import_submodules(name, submodules)
                    else:
                        import_submodules(name)
            else:
                auto_import[get_name(_obj, name)] = _obj
    else:
        for name in object_list:
            _obj = import_module(name)
            auto_import[getattr(_obj, "__name__", name)] = _obj
    for script in auto_scripts:
        exec(script, auto_import)
    return auto_import


def exec_init(manage_dict, context):
    for name, spec in manage_dict["shell"].get("init", {}).items():
        _obj = context.get(name, import_module(name))
        args = spec.get("args", []) if spec else []
        kwargs = spec.get("kwargs", {}) if spec else {}
        _obj(*args, **kwargs)


def exec_init_script(manage_dict, context):
    if "init_script" in manage_dict["shell"]:
        exec(manage_dict["shell"]["init_script"], context)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import atexit
import code
import copy
import json
import os
import sys
from importlib import import_module

import click
import yaml

from manage import __version__
from manage.template import default_manage_dict
from manage.auto_import import (
    exec_init,
    exec_init_script,
    import_objects
)
from manage.commands_collector import (
    load_command_sources,
    load_commands,
    load_groups
)

MANAGE_FILE = "manage.yml"
HIDDEN_MANAGE_FILE = ".{0}".format(MANAGE_FILE)
MANAGE_DICT = {}


def load_manage_dict(filename=None):
    manage_filename = None
    if not MANAGE_DICT:
        if filename:
            manage_filename = filename
        elif os.path.exists(MANAGE_FILE):
            manage_filename = MANAGE_FILE
        elif os.path.exists(HIDDEN_MANAGE_FILE):
            manage_filename = HIDDEN_MANAGE_FILE
        else:
            MANAGE_DICT.update(copy.deepcopy(default_manage_dict))
            MANAGE_DICT["shell"]["banner"]["message"] = (
                "WARNING: This is not a managed project\n"
                "\tPlease `exit()` and \n"
                "\trun `$ manage init`\n"
                "\tand edit `manage.yml` file with desired options"
            )
            MANAGE_DICT["shell"]["auto_import"]["display"] = False
        if manage_filename:

            loader = yaml.Loader
            if hasattr(yaml, 'FullLoader'):
                loader = yaml.FullLoader

            with open(manage_filename) as manage_file:
                MANAGE_DICT.update(yaml.load(manage_file, Loader=loader))

    return MANAGE_DICT


class Config(object):
    def __init__(self):
        self.filename = None
        self._manage_dict = None

    @property
    def manage_dict(self):
        if not self._manage_dict:
            self._manage_dict = load_manage_dict(self.filename)
        return self._manage_dict


@click.group(no_args_is_help=False)
def cli():
    """ Core commands wrapper """


@cli.command()
@click.option("--banner")
@click.option("--project-name")
@click.option("--hidden/--no-hidden", default=False)
@click.option("--backup/--no-backup", default=True)
def init(banner, project_name, hidden, backup):
    """Initialize a manage shell in current directory
        $ manage init --banner="My awesome app shell" --project-name "P1"
        initializing manage...
        creating manage.yml
    """
    manage_file = HIDDEN_MANAGE_FILE if hidden else MANAGE_FILE
    if os.path.exists(manage_file):
        if not click.confirm("Rewrite {0}?".format(manage_file)):
            return

        if backup:
            bck = ".bck_{0}".format(manage_file)
            with open(manage_file, "r") as source, open(bck, "w") as bck_file:
                bck_file.write(source.read())

    with open(manage_file, "w") as output:
        data = default_manage_dict
        if banner:
            data["shell"]["banner"]["message"] = banner
        if project_name:
            data["project_name"] = project_name
            data["help_text"] = "This is the {} inteactive shell".format(
                project_name
            )
        output.write(yaml.dump(data, default_flow_style=False))


@cli.command()
@click.option("--version", "-V", is_flag=True, default=False)
def debug(version=False):
    """Shows the parsed manage file -V shows version"""
    if version:
        print(__version__)
        return
    print(json.dumps(MANAGE_DICT, indent=2))


def create_shell(console, manage_dict=None, extra_vars=None, exit_hooks=None):
    """Creates the shell"""
    manage_dict = manage_dict or MANAGE_DICT
    _vars = globals()
    _vars.update(locals())
    auto_imported = import_objects(manage_dict)
    if extra_vars:
        auto_imported.update(extra_vars)
    _vars.update(auto_imported)
    msgs = []
    if manage_dict["shell"]["banner"]["enabled"]:
        msgs.append(
            manage_dict["shell"]["banner"]["message"].format(**manage_dict)
        )
    if auto_imported and manage_dict["shell"]["auto_import"]["display"]:
        auto_imported_names = [
            key
            for key in auto_imported.keys()
            if key not in ["__builtins__", "builtins"]
        ]
        msgs.append("\tAuto imported: {0}\n".format(auto_imported_names))

    banner_msg = u"\n".join(msgs)

    exec_init(manage_dict, _vars)
    exec_init_script(manage_dict, _vars)

    atexit_functions = [
        import_module(func_name)
        for func_name in manage_dict["shell"].get("exit_hooks", [])
    ]
    atexit_functions += exit_hooks or []
    for atexit_function in atexit_functions:
        atexit.register(atexit_function)

    if console == "ptpython":
        try:
            from ptpython.repl import embed

            embed({}, _vars)
        except ImportError:
            click.echo("ptpython is not installed!")
        return

    if console == "bpython":
        try:
            from bpython import embed

            embed(locals_=_vars, banner=banner_msg)
        except ImportError:
            click.echo("bpython is not installed!")
        return

    try:
        if console == "ipython":
            from IPython import start_ipython
            from traitlets.config import Config

            c = Config()
            c.TerminalInteractiveShell.banner2 = banner_msg

            # Append ipython extensions.
            c.InteractiveShellApp.extensions = []
            if 'ipython_extensions' in manage_dict['shell']:
                for extension in manage_dict['shell']['ipython_extensions']:
                    c.InteractiveShellApp.extensions.append(extension)

            # Append ipython exec_lines.
            c.InteractiveShellApp.exec_lines = []
            if 'ipython_exec_lines' in manage_dict['shell']:
                for exec_line in manage_dict["shell"]['ipython_exec_lines']:
                    c.InteractiveShellApp.exec_lines.append(exec_line)

            if manage_dict["shell"].get("ipython_auto_reload", True) is True:
                c.InteractiveShellApp.extensions.append("autoreload")
                c.InteractiveShellApp.exec_lines.append("%autoreload 2")
            start_ipython(argv=[], user_ns=_vars, config=c)
        else:
            raise ImportError
    except ImportError:
        if manage_dict["shell"]["readline_enabled"]:
            import readline
            import rlcompleter

            readline.set_completer(rlcompleter.Completer(_vars).complete)
            readline.parse_and_bind("tab: complete")
        shell = code.InteractiveConsole(_vars)
        shell.interact(banner=banner_msg)


@cli.command()
@click.option(
    "console",
    "--ipython",
    default=True,
    flag_value="ipython",
    help="Start with ipython console",
)
@click.option(
    "console", "--ptpython",
    flag_value="ptpython", help="Start with ptpython console"
)
@click.option(
    "console", "--bpython",
    flag_value="bpython", help="Start with bpython console"
)
@click.option(
    "console", "--python",
    flag_value="python", help="Start with python console"
)
def shell(console):
    """Runs a Python shell with context"""
    return create_shell(
        MANAGE_DICT.get("shell", {}).get("console", console), MANAGE_DICT
    )


def load_manage_dict_from_sys_args():
    single_option = [item for item in sys.argv if "--managefile=" in item]
    if single_option:
        filename = single_option[0].split("=")[-1]
        sys.argv.remove(single_option[0])
    elif "--managefile" in sys.argv:
        filename = sys.argv[sys.argv.index("--managefile") + 1]
        sys.argv.remove("--managefile")
        sys.argv.remove(filename)
    else:
        filename = None
    load_manage_dict(filename)


def init_cli(cli_obj, reset=False):

    if reset:
        global MANAGE_DICT
        MANAGE_DICT = {}

    sys.path.insert(0, ".")
    load_manage_dict_from_sys_args()

    # Get help_text
    cli.help = MANAGE_DICT.get(
        "help_text", "{project_name} Interactive shell!"
    ).format(**MANAGE_DICT)

    load_groups(cli, MANAGE_DICT)
    load_commands(cli, MANAGE_DICT)

    manager = click.CommandCollection(help=cli.help, no_args_is_help=False)
    manager.add_source(cli)

    load_command_sources(manager, MANAGE_DICT)
    for item in MANAGE_DICT.get("disabled", []):
        cli.commands.pop(item, None)
    return manager


def main():
    manager = init_cli(cli)
    return manager()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_manage
----------------------------------

Tests for `manage` module.
py.test should run with --boxed argument
"""
import os

from click.testing import CliRunner

from manage.cli import cli, init_cli

BASEPATH = os.path.abspath('.')


class TestManage(object):
    @classmethod
    def setup_class(cls):
        pass

    def go_to_example(self, path):
        path = os.path.join(BASEPATH, 'examples', path)
        os.chdir(path)
        init_cli(cli, reset=True)

    def test_command_line_interface(self):
        runner = CliRunner()
        result = runner.invoke(cli, ['--help'])
        assert result.exit_code == 0
        assert 'shell' in result.output
        help_result = runner.invoke(cli, ['--help'])
        assert help_result.exit_code == 0
        assert 'Show this message and exit.' in help_result.output

    def test_custom_command_names(self):
        runner = CliRunner()
        self.go_to_example('custom_command_names')
        result = runner.invoke(cli, ['--help'])
        assert result.exit_code == 0
        assert 'new_user' in result.output
        assert 'new_group' in result.output
        create_user = runner.invoke(cli, ['new_user', '--name=Bruno'])
        assert 'Creating the user Bruno' in create_user.output
        create_group = runner.invoke(cli, ['new_group', '--name=Users'])
        assert 'Creating the group Users' in create_group.output

    def test_custom_namespaces(self):
        runner = CliRunner()
        self.go_to_example('custom_namespaces')
        result = runner.invoke(cli, ['--help'])
        assert result.exit_code == 0
        assert 'bar_create-user' in result.output
        assert 'foo_new_group' in result.output
        create_user = runner.invoke(cli, ['bar_create-user', '--name=Bruno'])
        assert 'Creating the user Bruno' in create_user.output
        create_group = runner.invoke(cli, ['foo_new_group', '--name=Users'])
        assert 'Creating the group Users' in create_group.output

    def test_function_commands(self):
        runner = CliRunner()
        self.go_to_example('function_commands')
        result = runner.invoke(cli, ['--help'])
        assert result.exit_code == 0
        assert 'new_user' in result.output
        assert (
            "Error: Missing option '--password'"
            in runner.invoke(cli, ['new_user', '--name=Bruno']).output
        )
        create_user = runner.invoke(cli, ['new_user', '--name=Bruno', '--password=123'])
        assert 'Creating user Bruno' in create_user.output

    def test_inline_commands(self):
        runner = CliRunner()
        self.go_to_example('inline_commands')
        result = runner.invoke(cli, ['--help'])
        assert result.exit_code == 0
        assert 'clear_cache' in result.output
        clear_cache = runner.invoke(cli, ['clear_cache'])
        assert "'clean_days': 100" in clear_cache.output

        clear_cache = runner.invoke(cli, ['clear_cache', '--days=15'])
        assert "'clean_days': 15" in clear_cache.output

    def test_multiple_click_commands(self):
        runner = CliRunner()
        self.go_to_example('multiple_commands')
        result = runner.invoke(cli, ['--help'])
        assert result.exit_code == 0
        assert 'create-user' in result.output
        assert 'create-group' in result.output
        create_user = runner.invoke(cli, ['create-user', '--name=Bruno'])
        assert 'Creating the user Bruno' in create_user.output
        create_group = runner.invoke(cli, ['create-group', '--name=Users'])
        assert 'Creating the group Users' in create_group.output

    def test_namespaced_commands(self):
        runner = CliRunner()
        self.go_to_example('namespaced_commands')
        result = runner.invoke(cli, ['--help'])
        assert result.exit_code == 0
        assert 'user_create-user' in result.output
        assert 'group_new_group' in result.output
        create_user = runner.invoke(cli, ['user_create-user', '--name=Bruno'])
        assert 'Creating the user Bruno' in create_user.output
        create_group = runner.invoke(cli, ['group_new_group', '--name=Users'])
        assert 'Creating the group Users' in create_group.output

    def test_simple(self):
        runner = CliRunner()
        self.go_to_example('simple')
        result = runner.invoke(cli, ['--help'])
        assert result.exit_code == 0

    def test_simple_commands(self):
        runner = CliRunner()
        self.go_to_example('simple_commands')
        result = runner.invoke(cli, ['--help'])
        assert result.exit_code == 0
        assert 'create-user' in result.output
        assert 'create-group' in result.output
        create_user = runner.invoke(cli, ['create-user', '--name=Bruno'])
        assert 'Creating the user Bruno' in create_user.output
        create_group = runner.invoke(cli, ['create-group', '--name=Users'])
        assert 'Creating the group Users' in create_group.output

    def test_naval(self):
        runner = CliRunner()
        self.go_to_example('naval')

        # ship and mine is there?
        result = runner.invoke(cli, ['--help'])
        assert result.exit_code == 0
        assert 'ship' in result.output
        assert 'mine' in result.output

        # help for ship
        result = runner.invoke(cli, ['ship', '--help'])
        assert result.exit_code == 0
        assert 'new' in result.output
        assert 'move' in result.output
        assert 'shoot' in result.output

        # help for mine
        result = runner.invoke(cli, ['mine', '--help'])
        assert result.exit_code == 0
        assert 'set' in result.output
        assert 'remove' in result.output

        # ship new
        result = runner.invoke(cli, ['ship', 'new', 'FireStorm'])
        assert result.exit_code == 0
        assert result.output == 'Created ship FireStorm\n'

        # ship move
        result = runner.invoke(
            cli, ['ship', 'move', 'FireStorm', '12', '34', '--speed', '90']
        )
        assert result.exit_code == 0
        assert result.output == ('Moving ship FireStorm to 12.0,34.0 with speed 90\n')

        # ship shoot
        result = runner.invoke(cli, ['ship', 'shoot', 'FireStorm', '30', '45'])
        assert result.exit_code == 0
        assert result.output == 'Ship FireStorm fires to 30.0,45.0\n'

        # mine set
        result = runner.invoke(cli, ['mine', 'set', '10', '20'])
        assert result.exit_code == 0
        assert result.output == 'Set moored mine at 10.0,20.0\n'

        result = runner.invoke(cli, ['mine', 'set', '10', '20', '--drifting'])
        assert result.exit_code == 0
        assert result.output == 'Set drifting mine at 10.0,20.0\n'

        # mine remove
        result = runner.invoke(cli, ['mine', 'remove', '10', '20'])
        assert result.exit_code == 0
        assert result.output == 'Removed mine at 10.0,20.0\n'

    @classmethod
    def teardown_class(cls):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-


from setuptools import setup

def long_description():
    description = []
    for filename in ['README.md', 'HISTORY.md']:
        with open(filename, 'r') as fd:
            description.append(fd.read())

    return '\n\n'.join(description)

requirements = ['Click>=7.0', 'PyYAML>=3.11']

test_requirements = ['pytest>=2.9.2', 'pytest-xdist>=1.14']

setup(
    name='pymanage',
    version='0.1.15-dev0',
    description="Command Line Manager + Interactive Shell for Python Projects",
    long_description=long_description(),
    long_description_content_type='text/markdown',
    author="Bruno Rocha, Jorge Cardona",
    url='https://gitlab.com/jorgeecardona/pymanage',
    packages=['manage'],
    package_dir={'manage': 'manage'},
    entry_points={'console_scripts': ['manage=manage.cli:main']},
    include_package_data=True,
    install_requires=requirements,
    license="ISC license",
    zip_safe=False,
    keywords='manage',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8'
    ],
    test_suite='tests',
    tests_require=test_requirements,
)
